import math

import pymunk

from kivy.uix.widget import Widget
from kivy.graphics import *

import physics


class Bullet(Widget):
    radius = .2
    speed = 25.0

    max_bounces = 1
    timeout = 15

    def __init__(self, app):
        self.app = app
        Widget.__init__(self)
        with self.canvas:
            PushMatrix()
            Color(rgb=(1, 0, 1))
            self.position = Translate()
            app.graphics.draw_scene('models/bullet')
            PopMatrix()

        self.body = pymunk.Body()
        self.body.velocity_func = self.velocity_func
        self.body.parent = self
        self.collider = pymunk.Circle(self.body, self.radius)
        self.collider.mass = .5
        self.collider.elasticity = 1
        self.collider.collision_type = physics.BULLET
        self.collider.filter = physics.BULLET_FILTER

        self.bounce_count = 0
        self.timer = self.timeout

    def spawn(self, x, y, z, angle):
        self.bounce_count = 0
        self.timer = self.timeout

        self.position.y = y
        self.body.position = x, z
        rads = angle / 57.3
        self.body.velocity = math.cos(rads), math.sin(rads)
        if self.body not in self.app.physics.space.bodies:
            self.app.physics.space.add(self.body, self.collider)

        self.app.audio_manager.play_sound('bullet')

    def velocity_func(self, *args):
        speed = self.body.velocity.length
        if speed != Bullet.speed:
            self.body.velocity *= Bullet.speed / speed

    def update(self, dt):
        self.position.x = self.body.position.x
        self.position.z = self.body.position.y

        self.timer -= dt
        if self.timer <= 0:
            self.app.game_manager.remove_bullet(self)
