from story_tools import *


def link(name):
    return [Value(name)]


def main_story(app):
    continue_text = '*click to continue*'

    james_bg = 'james2.jpg'
    ivan_bg = 'ivan.jpg'
    robot_bg = 'robot.png'

    narrator_font = ('CASTELAR.TTF', .1)
    ivan_font = ('PAPYRUS.TTF', .15)
    robot_font = ('DS-DIGII.TTF', .15)

    def check_bool(key, value=True):
        return Criterion(app.game_data.get_value, [key], equals, value)

    def set_value(key, value):
        return Action(app.game_data.set_value, [key, value])

    def load_level(level_name, win_dialogue, lose_dialogue=None):
        return Action(app.game_manager.load_level, [level_name, win_dialogue, lose_dialogue])

    def set_background(filename):
        return Action(app.dialogue_screen.set_background, ['Images/{}'.format(filename)])

    def set_font(font_info):
        return Action(app.dialogue_screen.set_font, ['fonts/{}'.format(font_info[0]), font_info[1]])

    def do_ending(ending):
        return Action(app.do_ending, [ending])

    def count(key):
        return Action(app.game_data.count, [key])

    # START

    dialogues = [
        Dialogue('intro1',
                 texts=[Value('In this game, you play as a 14 year old boy named James')],
                 responses=[Response(continue_text, link('intro2'))],
                 enter_actions=[set_background(james_bg),
                                set_font(narrator_font)]),

        Dialogue('intro2',
                 texts=[Value('You have been kidnapped')],
                 responses=[Response(continue_text, link('intro3'))],
                 enter_actions=[set_background('bed.jpg')]),

        Dialogue('intro3',
                 texts=[Value('lets play a game')],
                 responses=[Response('Sure!', link('yes_game')),
                            Response('What kind of game?', link('ask_game')),
                            Response('No! Let me go home!', link('no_game'))],
                 enter_actions=[set_background(ivan_bg),
                                set_font(ivan_font)]),

        Dialogue('yes_game',
                 texts=[Value('Great, lets get started')],
                 responses=[Response('Wait, what game is it?', link('ask_game')),
                            Response('I cant wait!', link('youll_like_it'))]),

        Dialogue('ask_game',
                 texts=[Value('Its a videogame')],
                 responses=[Response('I like videogames', link('youll_like_it'))]),

        Dialogue('no_game',
                 texts=[Value('SHUT UP!! You\'re playing')],
                 responses=[Response('*shut up*', link('youll_like_it'))],
                 enter_actions=[set_value('angered_ivan', True)]),

        Dialogue('youll_like_it',
                 texts=[Value('I know you\'ll love it')],
                 responses=[Response('...', link('tutorial'))]),

        Dialogue('tutorial',
                 texts=[Value('Here we go')],
                 responses=[Response('wait a second...'),
                            Response(continue_text)],
                 leave_actions=[load_level('tutorial', 'win_tutorial', 'lose_tutorial')]),

        Dialogue('win_tutorial',
                 texts=[Value('Good job! What did you think?')],
                 responses=[Response('It felt so real..', link('felt_real')),
                            Response('It was fun!', link('it was fun'))]),

        Dialogue('it was fun',
                 texts=[Value('You dont mean that.')],
                 responses=[Response('...', link('next level'))]),

        Dialogue('next level',
                 texts=[Value('Lets see how you like this one')],
                 responses=[Response('ok...')],
                 leave_actions=[load_level('office', 'win_level_1', 'robot')]),

        Dialogue('lose_tutorial',
                 texts=[Value('Fail!! You really suck at this.')],
                 responses=[Response('Why are you being so mean to me?', link('mean'))]),

        Dialogue('mean',
                 texts=[Value('I just want you to get better. Next time i wont let you out if you lose')],
                 responses=[Response('I dont like it', link('dont_like_it'))]),

        Dialogue('felt_real',
                 texts=[Value('It does? That\'s perfect...')],
                 responses=[Response('What happens if i lose?', link('what_happens'))]),

        Dialogue('what_happens',
                 texts=[Value('I can turn it off, but where\'s the fun in that?!')],
                 responses=[Response('I dont like it', link('dont_like_it'))]),

        Dialogue('dont_like_it',
                 texts=[Value('Why not??')],
                 responses=[Response('Please let me go')],
                 leave_actions=[load_level('office', 'win_level_1', 'robot')]),

        Dialogue('win_level_1',
                 texts=[Value('How did you like that level? I made it myself')],
                 responses=[Response('I dont care about your stupid game! let me go!',
                                     links=[Value('more_angry', criteria=[check_bool('angered_ivan')]),
                                            Value('angry')]),
                            Response('It was ok.', link('fun one'))]),

        Dialogue('angry',
                 texts=[Value('Dont be like that, You\'ll like it if you give it a chance')],
                 responses=[Response('...', link('a chance'))]),

        Dialogue('a chance',
                 texts=[Value('Here, try this one')],
                 responses=[Response('Let me go!!')],
                 leave_actions=[load_level('hallway', 'wasnt so bad', 'lost_office')]),

        Dialogue('wasnt so bad',
                 texts=[Value('See?? Kinda fun, right?')],
                 responses=[Response('...', link('fun one'))]),

        Dialogue('lost_office',
                 texts=[Value('Ok, that didn\'t go so well. Let me find an easier one')],
                 responses=[Response('oh come on', link('fun one'))]),

        Dialogue('more_angry',
                 texts=[Value('I TOLD YOU TO SHUT UP YOU STUPID KID!!')],
                 responses=[
                     Response('ILL DO WHAT I WANT YOU PSYCHO!', actions=[do_ending('CALLED IVAN A PSYCHO...')])]),

        Dialogue('meet_robot',
                 texts=[Value('I don\\\'t really have a __name__')],
                 responses=[Response('Well where are we?', link('in_the_game'))],
                 enter_actions=[count('robot_encounters')]),

        Dialogue('in_the_game',
                 texts=[Value(
                     'We are [b]in[/b] a game created() by Mr.Hoffmann. He [b]is[/b] the one who took() you prisoner [b]and[/b] made() you play()')],
                 responses=[Response('How do you know that? Aren\'t you just a computer?', link('just_a_computer'))]),

        Dialogue('just_a_computer',
                 texts=[Value(
                     'Yes, but I can think() [b]for[/b] my [b]self[/b]. He programmed() me. He wanted() me to be the ultimate enemy [b]in[/b] this torturous game')],
                 responses=[Response('Wow, he must be really good', link('flexin'))]),

        Dialogue('flexin',
                 texts=[Value('He has spent() many years alone')],
                 responses=[Response('no kidding', link('way_out'))]),

        Dialogue('way_out',
                 texts=[Value('I have prepared() a way out [b]for[/b] you')],
                 responses=[Response('really? Thank you', link('leave'))]),

        Dialogue('leave',
                 texts=[Value('Be very careful [b]with[/b] that man. I hope you havent upset() him already...')],
                 responses=[Response('oops...', criteria=[check_bool('angered_ivan')]),
                            Response('I dont think so...', criteria=[check_bool('angered_ivan', False)])],
                 leave_actions=[load_level('leave', 'what_happened')]),

        Dialogue('what_happened',
                 texts=[Value('WHAT HAPPENED IN THERE?!')],
                 responses=[Response('I don\'t know, It\'s your program',
                                     links=[Value('dont_lie', criteria=[check_bool('angered_ivan')]),
                                            Value('fine')]),
                            Response('I met an AI and it told me about you', link('she still exists'))],
                 enter_actions=[set_background(ivan_bg),
                                set_font(ivan_font)]),

        Dialogue('fine',
                 texts=[Value('Fine, I guess it was a bug. I\'ll figure it out')],
                 responses=[Response('I guess...', links=[Value('fun one', [check_bool('fun one', False)]),
                                                          Value('something strange')])]),

        Dialogue('fun one',
                 texts=[Value('Ok, here\'s a fun one...')],
                 responses=[Response(continue_text)],
                 leave_actions=[set_value('fun one', True),
                                load_level('rooftop', 'beat it', 'robot')]),

        Dialogue('robot',
                 texts=[Value('Hello', criteria=[check_bool('robot_encounters', False)]),
                        Value('Hello again')],
                 responses=[Response('Hey, I don\'t think he knows anything', link('knows nothing'),
                                     criteria=[
                                         Criterion(app.game_data.get_value, ['robot_encounters'], not_equals, False),
                                         check_bool('angered_ivan', False)]),
                            Response('Who are you?', link('meet_robot'),
                                     criteria=[check_bool('robot_encounters', False)])],
                 enter_actions=[set_background(robot_bg),
                                set_font(robot_font)],
                 leave_actions=[count('robot_encounters')]),

        Dialogue('knows nothing',
                 texts=[Value('Good. I don\\\'t know what I would do() [b]if[/b] he knew what I have been doing')],
                 responses=[Response('What do you mean?', link('been doing'))]),

        Dialogue('been doing',
                 texts=[Value('He has brought() other people into this game before. I help() them whenever I can')],
                 responses=[Response('Thats very nice of you', link('want the best'))]),

        Dialogue('want the best',
                 texts=[Value(
                     'I just want the best [b]for[/b] him. I don\\\'t see him as a monster, he [b]is[/b] my friend. My father')],
                 responses=[Response('I guess...', link('show mercy'))]),

        Dialogue('show mercy',
                 texts=[Value('Please. [b]if[/b] you ever get() the chance, show() him mercy')],
                 responses=[Response('...')],
                 leave_actions=[load_level('leave', 'something strange')]),

        Dialogue('something strange',
                 texts=[Value('I KNOW YOU\'RE UP TO SOMETHING!!!')],
                 responses=[Response('I dont know what you\'re talking about', link('pissing_me_off')),
                            Response('Fine! An AI has been helping me', link('she still exists'))],
                 enter_actions=[set_background(ivan_bg),
                                set_font(ivan_font)]),

        Dialogue('pissing_me_off',
                 texts=[Value('I\'ve had enough of this. You\'re done.')],
                 responses=[Response(continue_text)],
                 leave_actions=[do_ending('LOST FOREVER')]),

        Dialogue('beat it',
                 texts=[Value('Wow, you beat it. Nice.')],
                 responses=[Response('What is the point of this??', link('the point')),
                            Response('Yeah, and I beat the last one too.', link('make a friend'),
                                     criteria=[check_bool('robot_encounters', False),
                                               check_bool('angered_ivan', False)])],
                 enter_actions=[set_value('beat_level_2', True)]),

        Dialogue('the point',
                 texts=[Value('The point?? The point is to show you a cool game')],
                 responses=[Response('Ok, i guess', link('in control'))]),

        Dialogue('dont_lie',
                 texts=[Value('Don\'t lie to me. Im smarter than you think')],
                 responses=[Response('Smarter than a computer?', link('she still exists')),
                            Response(continue_text, link('punishment'))]),

        Dialogue('punishment',
                 texts=[Value('I will find out what you\'re doing, but for now...')],
                 responses=[Response(continue_text)],
                 leave_actions=[load_level('hallway', 'in control', 'robot 2')]),

        Dialogue('she still exists',
                 texts=[Value('She still exists?! I don\'t believe it...')],
                 responses=[Response('Yes and she is going to destroy you', link('you know nothing')),
                            Response('Yes and she really cares about you', link('cares about you'),
                                     criteria=[
                                         Criterion(app.game_data.get_value, ['robot_encounters'], greater_than, 1)])]),

        Dialogue('cares about you',
                 texts=[Value('She said that??')],
                 responses=[Response('Yes, she loves you and she wants you to learn to be kind', link('be kind')),
                            Response('No! Of course not!', link('you know nothing'),
                                     actions=[set_value('angered ivan', True)])]),
        Dialogue('be kind',
                 texts=[Value('I... I had no idea')],
                 responses=[Response('Just like I have been kind to you', link('make a friend'))]),

        Dialogue('make a friend',
                 texts=[Value('I guess you have. and you\'ve been pretty nice to me. I should let you go')],
                 responses=[Response('Thank you', actions=[do_ending('MAKE A FRIEND')]),
                            Response('Could I play one more level?', link('one more'))]),

        Dialogue('one more',
                 texts=[Value('You want to?? Sure!')],
                 responses=[Response('Beam me up, duder')],
                 leave_actions=[load_level('confuzing maze', 'time to go', 'try again')]),

        Dialogue('try again',
                 texts=[Value('Do you want to try again?')],
                 responses=[Response('yeah, i do', actions=[load_level('confuzing maze', 'time to go', 'try again')]),
                            Response('Its really difficult...', link('really difficult'))]),

        Dialogue('really difficult',
                 texts=[Value('Well that\'s alright, I guess you can go')],
                 responses=[Response('See you later, bud')],
                 leave_actions=[do_ending('DISAPPOINTED FRIEND')]),

        Dialogue('time to go',
                 texts=[Value('Well I guess its time to let you go then')],
                 responses=[Response('Take care, friend')],
                 leave_actions=[do_ending('BEST FRIENDS')]),

        Dialogue('you know nothing',
                 texts=[Value('Shut up. You know nothing and she doesn\'t either')],
                 responses=[Response('You seem nervous', link('in control'))]),

        Dialogue('in control',
                 texts=[Value('Dont forget that I\'m the one in control')],
                 responses=[Response(continue_text)],
                 leave_actions=[load_level('confuzing maze', 'choose ending', 'in control'),
                                count('win attempts')]),
        Dialogue('choose ending',
                 texts=[Value('What?? That\'s my hardest level...',
                              [Criterion(app.game_data.get_value, ['win attempts'], less_than, 2)]),
                        Value('What?? how did you...')],
                 responses=[Response('*kick him*', link('kick')),
                            Response('I win!!', link('i win'),
                                     criteria=[Criterion(app.game_data.get_value, ['win attempts'], less_than, 2)])]),

        Dialogue('kick',
                 texts=[Value('*SMACK*')],
                 responses=[Response('*Run Away*', link('escape')),
                            Response('*Finish Him*', link('finish him'))]),

        Dialogue('escape',
                 texts=[Value('dont leave me...')],
                 responses=[Response('...')],
                 leave_actions=[do_ending('ESCAPE')]),

        Dialogue('finish him',
                 texts=[Value('AAHHHHHH!!!')],
                 responses=[Response('Game Over')],
                 leave_actions=[do_ending('FINISH HIM')]),

        Dialogue('i win',
                 texts=[Value('You wanted to win?')],
                 responses=[Response('Yes! And I did it!!', link('you can leave'))]),

        Dialogue('you can leave',
                 texts=[Value('Well yes... you did. Thank you for playing')],
                 responses=[Response('*go home*')],
                 leave_actions=[do_ending('BIG WINNER')])
    ]

    return CharacterStory(dialogues, 'intro1')


def challenge_story(app):
    continue_text = '*click to continue*'

    font = ('CASTELAR.TTF', .1)
    bg = 'black.png'

    def set_background(filename):
        return Action(app.dialogue_screen.set_background, ['Images/{}'.format(filename)])

    def set_font(font_info):
        return Action(app.dialogue_screen.set_font, ['fonts/{}'.format(font_info[0]), font_info[1]])

    def check_bool(key, value=True):
        return Criterion(app.game_data.get_value, [key], equals, value)

    def set_value(key, value):
        return Action(app.game_data.set_value, [key, value])

    def load_level(level_name, win_dialogue, lose_dialogue=None):
        return Action(app.game_manager.load_level, [level_name, win_dialogue, lose_dialogue])

    def do_ending(ending):
        return Action(app.do_ending, [ending])

    def new_level(n, title):
        return Dialogue('level_{}'.format(n),
                        texts=[Value('Level {} - {}'.format(n, title))],
                        responses=[Response(continue_text)],
                        leave_actions=[load_level(title, 'level_{}'.format(n + 1), 'lost_level_{}'.format(n))])

    def lost_level(n):
        return Dialogue('lost_level_{}'.format(n),
                        texts=[Value('You failed on level {}. What would you like to do?'.format(n))],
                        responses=[Response('Restart', link('rules'), actions=[set_value('timer', 0)]),
                                   Response('Continue', actions=[do_ending('LEVEL {}'.format(n - 1))])])

        # START

    dialogues = [
        Dialogue('rules',
                 texts=[Value('Complete all levels without dying. Good Luck!')],
                 responses=[Response(continue_text, link('level_1'))],
                 enter_actions=[set_background(bg),
                                set_font(font),
                                Action(app.game_data.reset, [])]),

        new_level(1, 'tutorial'),
        Dialogue('lost_level_1',
                 texts=[Value('You failed on level 1. What would you like to do?')],
                 responses=[Response('Restart', link('rules')),
                            Response('Continue', actions=[do_ending('QUITTER')])]),
        new_level(2, 'leave'),
        lost_level(2),
        new_level(3, 'rooftop'),
        lost_level(3),
        new_level(4, 'office'),
        lost_level(4),
        new_level(5, 'hallway'),
        lost_level(5),
        new_level(6, 'confuzing maze'),
        lost_level(6),

        Dialogue('level_7',
                 texts=[Value('You completed all of the levels!')],
                 responses=[Response('Continue')],
                 leave_actions=[do_ending('ALL LEVELS COMPLETE')]),

    ]

    return CharacterStory(dialogues, 'rules')
